@@ -149,7 +149,7 @@ class ActionBag(QObject):
                 QKeySequence.Quit, 'application-exit')
         self.__findAction = self.createAction(self.tr('&Find'), QKeySequence.Find,
                 'edit-find')
-        self.__filterAction = self.createAction(self.tr('F&ilter'))
+        self.__filterAction = self.createAction(self.tr('F&ilter'), 'Ctrl+J')
         self.__settingsAction = self.createAction(self.tr('Settings'),
                 QKeySequence.Preferences, 'preferences-other')
         self.__parentWidthAction = self.createAction(self.tr('Parent'))
