@@ -180,6 +180,7 @@ class CurrentNodeCommand(SimpleCommand):
         SimpleCommand.__init__(self)
         self.__node = node
 
+        
 class ShutdownCommand(SimpleCommand):
     '''Indicates that collaboration is finished'''
 
@@ -192,3 +193,40 @@ class ShutdownCommand(SimpleCommand):
         '''
         SimpleCommand.__init__(self)
         self.__reason = reason
+
+
+class AffiliationCommand(SimpleCommand):
+    '''Contains affiliation for participant'''
+
+    readable('entity', 'affiliation')
+
+    def __init__(self, entity, affiliation):
+        '''
+        @type entity: str
+        @type affiliation: str
+        '''
+        SimpleCommand.__init__(self)
+        self.__entity = entity
+        self.__affiliation = affiliation
+
+
+class ConfigurationCommand(SimpleCommand):
+    '''Contains application configuration options'''
+
+    readable('config')
+
+    def __init__(self, config = None):
+        '''
+        Constructor
+        '''
+        SimpleCommand.__init__(self)
+        self.__config = dict() if config is None else config
+
+    def setOption(self, name, value):
+        """
+        Set single configuration option
+        @param name: Option name
+        @param value: Option value
+        """
+        self.__config[name] = value
+
