@@ -31,7 +31,7 @@ from PyQt4.QtCore import QSize, Qt
 from hivemind.dialogs import EditNodeDialog, EditLabelDialog, IconDialog, \
 AboutDialog, EditEdgeDialog, SettingsDialog, NetworkConnectionDialog, \
 AffiliationDialog, NetworkSettingsDialog, NetworkClientDialog
-from hivemind.gui_widgets import FindToolBar, FilterToolBar
+from hivemind.gui_widgets import FindToolBar, FilterToolBar, HivemindToolBar, StatusAction
 from hivemind.attribute import adaptCallable
 import hivemind.__version__ as __version__
 
@@ -347,7 +347,7 @@ class DesktopNetworkConnectionDialog(NetworkConnectionDialog):
         NetworkConnectionDialog.__init__(self, parent)
 
 
-class DesktopFilterToolBar(QToolBar, FilterToolBar):
+class DesktopFilterToolBar(HivemindToolBar, FilterToolBar):
     '''
     Filter tool bar for Desktop platform
     @author: Oleg Kandaurov
@@ -358,21 +358,29 @@ class DesktopFilterToolBar(QToolBar, FilterToolBar):
         @type parent: QWidget
         '''
         FilterToolBar.__init__(self, controller)
-        QToolBar.__init__(self, parent)
+        HivemindToolBar.__init__(self, parent)
         self.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
-        FilterToolBar._createComponents(self)
+        self._createComponents()
         self._initComponents()
-        FilterToolBar._createLayout(self)
+
+    def _createComponents(self):
+        '''Create components'''
+        #pylint: disable=W0201
+        self._modeAction = StatusAction('Filtration mode', 'filtermode', 'text', self, 32)
+        FilterToolBar._createComponents(self)
 
     def _initComponents(self):
         '''Init components'''
-        self._closeButton.clicked.connect(self.close)
+        self._closeButton.clicked.connect(self._close)
         self._closeButton.setIcon(QIcon.fromTheme('window-close'))
-        self._findLineEdit.setFixedWidth(self._findLineEdit.sizeHint().width())
-        self.addWidget(self._findWidget)
+        self._closeButton.setText(self.tr('Close'))
+        self._modeButton.setText(self.tr('Switch mode'))
+        self._filterLineEdit.setFixedWidth(self._filterLineEdit.sizeHint().width())
+        self._setTextMode()
+        self.addWidget(self._filterWidget)
 
 
-class DesktopFindToolBar(QToolBar, FindToolBar):
+class DesktopFindToolBar(HivemindToolBar, FindToolBar):
     '''
     Find tool bar for Desktop platform
     @author: Eldar Mamedov, Oleg Kandaurov
@@ -382,7 +390,7 @@ class DesktopFindToolBar(QToolBar, FindToolBar):
         @type controller: MainWindowController
         @type parent: QWidget
         '''
-        QToolBar.__init__(self, parent)
+        HivemindToolBar.__init__(self, parent)
         FindToolBar.__init__(self, controller)
         self.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
         self._createComponents()
