@@ -68,6 +68,21 @@ class DesktopMainView(MainView):
             if item is None:
                 self.mapClicked.emit()
 
+    def wheelEvent(self, event):
+        '''
+        Wheel event handler
+        Changes scale of the map
+
+        @type event: QWheelEvent
+        '''
+        if event.modifiers() == Qt.ControlModifier:
+            if event.delta() > 0:
+                self._zoomIn()
+            else:
+                self._zoomOut()
+        else:
+            MainView.wheelEvent(self, event)
+
 
 class DesktopMainWindow(AbstractMainWindow):
     '''
