@@ -30,7 +30,7 @@ QVBoxLayout, QFormLayout, QAbstractItemView, QTableView, QHeaderView, QHBoxLayou
 QComboBox, QPushButton, QFontComboBox, QAction, QFontDatabase, QColorDialog, QTextCharFormat, \
 QFont, QPalette, QBrush, QTextCursor, QGridLayout, QCheckBox, QGraphicsView, \
 QTabWidget, QWidget, QSpinBox, QScrollArea, QToolBar
-from PyQt4.QtCore import SIGNAL, QRegExp, Qt, QSize, QCoreApplication, QObject
+from PyQt4.QtCore import SIGNAL, QRegExp, Qt, QSize, QCoreApplication, QObject, pyqtSignal
 from hivemind.attribute import readable, adaptCallable
 from hivemind.gui_widgets import SearchingTrMixin, SpecialTextEdit, SettingsCreator, \
 ScalingWidget, IconList, IconTable, LineEditBinder, ColorLabel
@@ -1329,6 +1329,8 @@ class NetworkErrorDialog(QMessageBox):
     @author: Oleg Kandaurov
     '''
 
+    performAction = pyqtSignal(str)
+
     def __init__(self, action, parent = None):
         '''
         Constructor
@@ -1356,15 +1358,19 @@ class NetworkErrorDialog(QMessageBox):
         '''
         if not fatal:
             actionButton = QPushButton(self.tr('Disconnect'))
-            actionButton.clicked.connect(self.__action)
+            actionButton.clicked.connect(self._performAction)
             self.addButton(actionButton, QMessageBox.ActionRole)
         else:
-            self.__action()
+            self._performAction()
         self.setText(info)
         self.setInformativeText(reason)
         self.setDetailedText(details)
         self.exec_()
 
+    def _performAction(self):
+        '''Send signal with action name'''
+        self.performAction.emit(self.__action)
+
 
 class EditHyperlinkDialog(CommonDialog):
     '''
