@@ -25,7 +25,7 @@ View delegates for core objects
 '''
 
 import math
-from PyQt4.QtCore import QRectF, QPointF, Qt, QObject, pyqtSignal, QCoreApplication, QString
+from PyQt4.QtCore import QRectF, QPointF, Qt, QObject, pyqtSignal, QCoreApplication
 from PyQt4.QtGui import QGraphicsScene, QGraphicsItem, QAction, QPen, \
 QPainterPath, QPolygonF, QIcon, QTextDocument, QPainter, QGraphicsView
 import hivemind.resource as resource
@@ -36,16 +36,15 @@ from hivemind.core import MindMap, Node
 class MindMapDelegate(QGraphicsScene):
     '''
     MindMap delegate
-    
+
     Warning! C{currentNode} field is self-encapsulated!
     You must not use direct invocation (C{self.__currentNode})
     inside the class!
-    
+
     @authors: Andrew Golovchenko, Andrew Vasilev, Alexander Kulikov
     '''
 
-    readable('rootDelegate', 'mindMap')
-    writable('currentLayerDepth')
+    readable('rootDelegate', 'mindMap', 'nodeMapCreator')
 
     mapChanged = pyqtSignal()
     curMapChanged = pyqtSignal()
@@ -67,7 +66,6 @@ class MindMapDelegate(QGraphicsScene):
         delegate = self.__nodeToDelegate[currentNode]
         delegate.selected = True
         delegate.ensureVisible(delegate.textBoundingRect())
-        self.__currentLayerDepth = currentNode.depth()
         self.currentNodeChanged.emit(self.__currentNode)
 
     currentNode = property(_getCurrentNode, _setCurrentNode, None, 'Current node')
@@ -79,9 +77,9 @@ class MindMapDelegate(QGraphicsScene):
         self.__actionBag = actionBag
         self.__nodeToDelegate = {}
         self.__rootDelegate = None
-        self.__currentLayerDepth = None
         self.__currentNode = None
         self.__mindMap = None
+        self.__nodeMapCreator = NodeMapCreator()
         # used as 'switch' operator in processNotification() method
         self.__signalHandlers = {'childNodeAdded' : self.addNodeDelegate,
                                  'childNodeRemoved' : self.removeNodeDelegate,
@@ -119,9 +117,8 @@ class MindMapDelegate(QGraphicsScene):
         #pylint: disable=C0103
         self._addDelegatesForNodeSubtree(node)
         self.positionNodes()
-        if not node.parent.folded:
+        if self.__nodeToDelegate[node].isVisible():
             self.currentNode = node
-        self.__currentLayerDepth = node.depth()
 
     def _addDelegatesForNodeSubtree(self, root):
         '''
@@ -141,14 +138,13 @@ class MindMapDelegate(QGraphicsScene):
     def nodeBoundsChanged(self, node):
         '''
         This method called, then signal for node bounds changed has come
-        @param node: node was modify
+        @param node: modified node
         @type node: Node
         '''
-        if node.children:
-            visible = False if node.folded is not None else True
-            if self.__nodeToDelegate[node.children[0]].isVisible() != visible:
-                for child in node.children:
-                    self.__nodeToDelegate[child].setVisible(visible)
+        visible = False if node.folded is not None else True
+        for child in node.children:
+            if self.__nodeToDelegate[child].isVisible() != visible:
+                self.__nodeToDelegate[child].setVisible(visible)
         self.__nodeToDelegate[node].updateNodeBounds(False)
         self.positionNodes(node)
 
@@ -178,7 +174,8 @@ class MindMapDelegate(QGraphicsScene):
         '''
         delegate = self.__nodeToDelegate[node]
         if node.isAncestorOf(self.__currentNode):
-            self.currentNode = delegate.parentItem().node
+            self.__nodeToDelegate[node.parent].setLastChild(None)
+            self.currentNode = node.parent
         self._removeNodeDelegateSubtree(node)
         self.removeItem(delegate)
         self.positionNodes()
@@ -198,7 +195,7 @@ class MindMapDelegate(QGraphicsScene):
 
         @param node: transfered node
         @type node: Node
-        
+
         @param dest: destination node
         @type dest: Node
         '''
@@ -209,12 +206,11 @@ class MindMapDelegate(QGraphicsScene):
         # node relocation require recalculate position of all nodes
         roamingDelegate.updateNodeBounds()
         self.positionNodes()
-        self.__currentLayerDepth = node.depth()
 
     def setMap(self, mindMap):
         '''
         Load new mind map and show it
-        
+
         @param mindMap: mind map to be shown
         @type mindMap: MindMap
         '''
@@ -231,7 +227,6 @@ class MindMapDelegate(QGraphicsScene):
         self.createNodeDelegates(self.__rootDelegate)
         self.positionNodes()
         self.currentNode = mindMap.root
-        self.__currentLayerDepth = 0
         if len(self.views()):
             self.views()[0].setViewportUpdateMode(QGraphicsView.SmartViewportUpdate)
         self.mapChanged.emit()
@@ -251,7 +246,7 @@ class MindMapDelegate(QGraphicsScene):
     def createNodeDelegates(self, parentNodeDelegate):
         '''
         Create node delegates from parent
-        
+
         @type parentNodeDelegate: NodeDelegate
         '''
         QCoreApplication.instance().processEvents()
@@ -423,88 +418,93 @@ class MindMapDelegate(QGraphicsScene):
         '''
         return position.y() < item.textRectOnScene().center().y()
 
-    def _selectNextLeftRightNode(self, leftSubtree):
-        '''
-        Switch currentNode to left or to right node
-        @type leftSubtree: boolean
-        @param leftSubtree: select left node if true.
-        '''
-        nextNode = None
-        if self.currentNode.left == leftSubtree or self.currentNode.parent is None:
-            if self.currentNode.children and not self.currentNode.folded:
-                #node has children and is not folded
-                currentDelegate = self.getDelegate(self.currentNode)
-                position = currentDelegate.textRectOnScene().center()
-                if self.currentNode.parent is None:
-                    position.setX(position.x() + (-1 if leftSubtree else 1))
-                above, below = self._neighbourDelegates(position, currentDelegate)
-                nearest = MindMapDelegate.nearestDelegateByOrdinate(position, above,
-                        below)
-                nextNode = nearest.node
-        else:
-            #select parent
-            nextNode = self.currentNode.parent
-        if nextNode:
-            self.currentNode = nextNode
-            self.__currentLayerDepth = nextNode.depth()
-
-    def _createNodesLayer(self, root, maxDepth, leftSubtree, selectedNodes):
-        '''
-        Find nodes with the depth equal to maxDepth in the both left and right subtrees
-        and put them to the list.
-        @type root: Node
-        @type maxDepth: int
-        @type leftSubtree: bool
-        @type selectNodes: list
-        '''
-        if not root.folded and maxDepth > 0:
-            for node in root.children:
-                if node.left == leftSubtree:
-                    self._createNodesLayer(node, maxDepth - 1, leftSubtree, selectedNodes)
-        if root.folded or maxDepth == 0 or len(root.children) == 0:
-            selectedNodes.append(root)
-
     def _selectRootNode(self):
         '''Select root node'''
         self.currentNode = self.rootDelegate.node
 
     def _selectLeftNode(self):
         '''Select left node'''
-        self._selectNextLeftRightNode(True)
+        self._selectHorizontalNode(left = True)
 
     def _selectRightNode(self):
         '''Select right node'''
-        self._selectNextLeftRightNode(False)
+        self._selectHorizontalNode(left = False)
 
     def _selectUpNode(self):
         '''Select up node'''
-        nodes = []
-        self._createNodesLayer(self.rootDelegate.node,
-                self.__currentLayerDepth, self.currentNode.left, nodes)
-        for node in nodes:
-            if self.currentNode.isAncestorOf(node) or self.currentNode == node:
-                index = nodes.index(node)
-                if index > 0:
-                    self.currentNode = nodes[index - 1]
-                return
+        self._selectVerticalNode(up = True)
 
     def _selectDownNode(self):
         '''Select down node'''
-        nodes = []
-        self._createNodesLayer(self.rootDelegate.node,
-                self.__currentLayerDepth, self.currentNode.left, nodes)
+        self._selectVerticalNode(up = False)
+
+    def _sortNodesByDistance(self, nodes, currentNode):
+        '''
+        Create list of nodes sorted by y-distance relative to the current node
+        @param nodes: list of nodes for sorting
+        @type currentNode: Node
+        @return: sorted list of nodes
+        '''
+        def compare(first, second):
+            '''Compare nodes by y position'''
+            firstPos = self.getDelegate(first).textRectOnScene().center().y()
+            secondPos = self.getDelegate(second).textRectOnScene().center().y()
+            return int(abs(firstPos - curPos) - abs(secondPos - curPos))
+
+        currentDelegate = self.getDelegate(currentNode)
+        curPos = currentDelegate.textRectOnScene().center().y()
+        return sorted(nodes, cmp = compare)
+
+    def _selectHorizontalNode(self, left):
+        '''
+        Select node in horizontal direction
+        @param left: select left node if True.
+        '''
+        if self.currentNode.left != left and self.currentNode.parent is not None:
+            parentNodeDelegate = self.__nodeToDelegate[self.currentNode.parent]
+            parentNodeDelegate.setLastChild(self.currentNode)
+            self.currentNode = self.currentNode.parent
+            return
+        if not self.currentNode.children or self.currentNode.folded:
+            return
+        lastChild = self.__nodeToDelegate[self.currentNode].getLastChild(left)
+        if lastChild:
+            self.currentNode = lastChild
+            return
+        self._selectNearestChidlNode(left)
+
+    def _selectNearestChidlNode(self, left):
+        '''
+        Select nearest child node of current selected node
+        @param left: if true search node in left subtree, search in right subtree otherwise
+        '''
+        nodeMap = self.__nodeMapCreator.createNodeMap(self.rootDelegate.node)
+        nodes = nodeMap[self.currentNode.depth() + (-1 if left else 1)]
+        nodes = self._sortNodesByDistance(nodes, self.currentNode)
         for node in nodes:
-            if self.currentNode.isAncestorOf(node) or self.currentNode == node:
-                index = nodes.index(node)
-                if index + 1 < len(nodes):
-                    self.currentNode = nodes[index + 1]
+            if not self.currentNode.isAncestorOf(node): continue
+            if self.getDelegate(node).isVisible():
+                self.currentNode = node
                 return
 
+    def _selectVerticalNode(self, up):
+        '''
+        Select node in vertical direction
+        @param up: if True select node above otherwise below
+        '''
+        nodeMap = self.__nodeMapCreator.createNodeMap(self.rootDelegate.node)
+        nodes = nodeMap[self.currentNode.depth()]
+        newPos = nodes.index(self.currentNode) + (-1 if up else 1)
+        if newPos in range(0, len(nodes)):
+            nextNode = nodes[newPos]
+            if self.getDelegate(nextNode).isVisible():
+                self.currentNode = nextNode
+
 
 class NodeDelegate(QGraphicsItem):
     '''
     Node delegate
-    
+
     @authors: Andrew Golovchenko, Andrew Vasilev, Alexander Kulikov
     '''
     readable('node', 'verticalSpace')
@@ -540,10 +540,10 @@ class NodeDelegate(QGraphicsItem):
     def __init__(self, node, parent = None):
         '''
         Initialize delegate with specified node
-        
+
         @param node: delegated Node object
         @type node: Node
-        @param parent: parent object of this delegate 
+        @param parent: parent object of this delegate
         @type parent: NodeDelegate
         '''
         self.__node = node
@@ -552,6 +552,7 @@ class NodeDelegate(QGraphicsItem):
         self.__selected = False
         self.__positionChanged = True
         self.__textBoundingRect = None
+        self.__lastChild = None if node.parent else [None, None]
         self.__boundingRect = None
         QGraphicsItem.__init__(self, parent)
         self.__edge = Edge(self.__node, self)
@@ -623,7 +624,6 @@ class NodeDelegate(QGraphicsItem):
         for child in self._childDelegates():
             child.updateSubtree()
 
-
     def _computeSoloVerticalSpace(self):
         '''
         Compute minimum vertical space needed for the current node
@@ -937,7 +937,7 @@ class NodeDelegate(QGraphicsItem):
         top = min(textBounds.y(), -self._labelOverhead())
         width = textBounds.width() + abs(self.linkPointOffset())
         if self.__node.folded and self.__node.left: left -= NodeDelegate.__FOLD_MARK_SIZE
-        height = max(textBounds.height(), self.__node.labelSize.height())
+        height = textBounds.height() + self._labelOverhead()
         if self.__node.folded:
             width += NodeDelegate.__FOLD_MARK_SIZE
             if self.__node.effectiveStyle == 'fork': height += NodeDelegate.__FOLD_MARK_SIZE
@@ -1009,6 +1009,39 @@ class NodeDelegate(QGraphicsItem):
         offset = labelWidth + NodeDelegate.__LABEL_MARGIN * 2.0
         return offset if self.__node.left else -offset
 
+    def getLastChild(self, leftDirection):
+        '''
+        Retrieve last selected child on specified direction
+        @param leftDirection: indication whether child was on the left or right
+        @type leftDirection: bool
+        @return: link to the child node
+        @rtype: Node
+        '''
+        lastChild = None
+        if self.__node.parent is None:
+            if leftDirection:
+                lastChild = self.__lastChild[0]
+            else:
+                lastChild = self.__lastChild[1]
+        else:
+            lastChild = self.__lastChild
+        if lastChild not in self.__node.children: lastChild = None
+        return lastChild
+
+    def setLastChild(self, child):
+        '''
+        Set this node as a last child
+        @param child: child node that was selected before this node
+        @type child: Node
+        '''
+        if self.__node.parent is None:
+            if child.left:
+                self.__lastChild[0] = child
+            else:
+                self.__lastChild[1] = child
+        else:
+            self.__lastChild = child
+
 
 class Edge(QGraphicsItem):
     '''
@@ -1189,103 +1222,138 @@ class NodeLocation(object):
         return self.__position is not None or self.__right is not None
 
 
-class Finder(QObject):
+class NodeMapCreator(QObject):
     '''
-    Find nodes that match a criteria
+    Create and process various node containers
     @author: Oleg Kandaurov
     '''
 
     def __init__(self):
         '''Constructor'''
         QObject.__init__(self)
-        self.__processCondition = None
-        self.__nodeMap = {}
 
-    def findNodes(self, node, processCondition):
+    def createNodeMap(self, rootNode):
         '''
-        Find all children of given node that match the condition
-        @param node: Find from the Node
-        @param processCondition: Function that provide a condition
-        @return: list of Node
+        Create dict containing all node of the map.
+        Format is {0: [n0], 1: [n1, n2], -1: [n3, n4], 2:[...] ...}
+        @param rootNode: Root node of the mindmap
+        @type rootNode: Node
+        @return: dict of Nodes
         '''
-        self.__processCondition = processCondition
-        self.__nodeMap = {}
-        self._find(node, 0)
+        nodeMap = {}
+        self._extendNodeMap(rootNode, 0, nodeMap)
+        return nodeMap
 
-    def createSortedList(self):
+    @classmethod
+    def includeParentNodes(cls, nodeMap):
         '''
-        Create list of found nodes sorted from left to right
-        @return: list of found nodes
+        Include parents of all nodes in the nodeMap
+        @param nodeMap: Node map that need to be processed
         '''
-        foundNodes = []
-        keys = self.__nodeMap.keys()
+        depths = nodeMap.keys()
+        depths.sort(lambda x, y: cmp (abs(y), abs(x))) # 4 -3 3 traverse from edges of mindmap
+        for depth in depths:
+            for node in nodeMap[depth]:
+                upperLevel = depth + 1 if node.left else depth - 1
+                if upperLevel not in depths:
+                    nodeMap[upperLevel] = []
+                if node.parent not in nodeMap[upperLevel]:
+                    nodeMap[upperLevel].append(node.parent)
+
+    @classmethod
+    def createSortedNodeList(cls, nodeMap):
+        '''
+        Create list of nodes sorted from left to right
+        @param nodeMap:
+        @return: list of nodes
+        '''
+        nodes = []
+        keys = nodeMap.keys()
         keys.sort()
         for key in keys:
-            foundNodes.extend(self.__nodeMap[key])
-        return foundNodes
+            nodes.extend(nodeMap[key])
+        return nodes
 
-    def includeParentNodes(self):
-        '''Include parents nodes of found nodes'''
-        depths = self.__nodeMap.keys()
-        depths.remove(0)
-        depths.sort(lambda x, y: cmp (abs(y), abs(x)))
-        for depth in depths:
-            for node in self.__nodeMap[depth]:
-                upperLevel = depth + 1 if node.left else depth - 1
-                if node.parent not in self.__nodeMap[upperLevel]:
-                    self.__nodeMap[upperLevel].append(node.parent)
-
-    def _find(self, node, depth):
+    @classmethod
+    def createFilteredNodeMap(cls, nodeMap, condition):
         '''
-        Recursive function to find nodes
+        Create new node map from the given one
+        @param nodeMap: node map
+        @param condition: function that returns True if node matches a condition
+        @return: filtered node map
+        '''
+        filteredNodeMap = {}
+        for depth in nodeMap:
+            for node in nodeMap[depth]:
+                if condition(node):
+                    if depth not in filteredNodeMap:
+                        filteredNodeMap[depth] = []
+                    filteredNodeMap[depth].append(node)
+        return filteredNodeMap
+
+    def _extendNodeMap(self, node, depth, nodeMap):
+        '''
+        Append nodes of given depth to the node map
         @type node: Node
         @type depth: int
         '''
-        if depth not in self.__nodeMap:
-            self.__nodeMap[depth] = []
-        if self.__processCondition(node):
-            self.__nodeMap[depth].append(node)
+        if depth not in nodeMap:
+            nodeMap[depth] = []
+        nodeMap[depth].append(node)
         for childrenNode in node.children:
             deeperLevel = depth - 1 if childrenNode.left else depth + 1
-            self._find(childrenNode, deeperLevel)
+            self._extendNodeMap(childrenNode, deeperLevel, nodeMap)
 
 
 class MindMapFilter(QObject):
     '''
-    Filters mind map
+    Filter mind map
     @author: Oleg Kandaurov
     '''
 
+    writable('mode', 'text', 'icons')
+
     def __init__(self, controller):
         '''Constructor'''
         QObject.__init__(self)
         self.__controller = controller
         self.__mindMapScene = controller.mindMapScene
-        self.__finder = Finder()
+        self.__nodemapCreator = self.__mindMapScene.nodeMapCreator
         self.__foundNodes = []
         self.__filterMode = False
-        self.__targetText = QString()
+        self.__text = None
+        self.__icons = []
+        self.__mode = None
         self.__mindMapScene.curMapChanged.connect(self.refresh)
         self.__mindMapScene.mapChanged.connect(self.refresh)
 
-    def setTargetText(self, targetText):
-        '''Setter'''
-        self.__targetText = targetText
-
-    def _condition(self, node):
+    def _textCondition(self, node):
         '''
         Decide whether node match a condition or not
         @param node: Node to be inspected
         @return: True if node matches a condition
         '''
-        if not len(self.__targetText):
-            return False
+        if self.__text is None or not len(self.__text):
+            return True
         nodeText = QTextDocument()
         nodeText.setHtml(node.text)
-        if self.__targetText.toLower() in nodeText.toPlainText().toLower():
+        if unicode(self.__text.toLower()) in unicode(nodeText.toPlainText().toLower()):
             return True
         return False
 
+    def _iconCondition(self, node):
+        '''
+        Decide whether node match a condition or not
+        @param node: Node to be inspected
+        @return: True if node matches a condition
+        '''
+        if not len(self.__icons):
+            return True
+        for icon in node.icons:
+            if icon in self.__icons:
+                return True
+        return False
+
     def _filter(self, node, isVisible):
         '''
         Filter given node
@@ -1324,18 +1392,20 @@ class MindMapFilter(QObject):
         if not filterMode:
             self._filterNodes()
         else:
-            self._updateFilteredNodes()
-
-    def _updateFilteredNodes(self):
-        '''Find nodes and filter it'''
-        self.__finder.findNodes(self.__mindMapScene.rootDelegate.node, self._condition)
-        self.__finder.includeParentNodes()
-        self.__foundNodes = self.__finder.createSortedList()
-        self._filterNodes()
+            self.refresh()
 
     def refresh(self):
-        '''Refresh all filtered nodes'''
-        self._updateFilteredNodes()
+        '''Find nodes and filter it'''
+        nodeMap = self.__nodemapCreator.createNodeMap(self.__mindMapScene.rootDelegate.node)
+        if self.__mode == 'text':
+            filteredNodeMap = self.__nodemapCreator.createFilteredNodeMap(nodeMap,
+                    self._textCondition)
+        else:
+            filteredNodeMap = self.__nodemapCreator.createFilteredNodeMap(nodeMap,
+                    self._iconCondition)
+        self.__nodemapCreator.includeParentNodes(filteredNodeMap)
+        self.__foundNodes = self.__nodemapCreator.createSortedNodeList(filteredNodeMap)
+        self._filterNodes()
 
 
 class NodeFinder(QObject):
@@ -1351,11 +1421,11 @@ class NodeFinder(QObject):
         @type controller: MainWindowController
         '''
         QObject.__init__(self)
-        self.__finder = Finder()
         self.__foundNodes = []
         self.__index = -1
         self.__controller = controller
         self.__mindMapScene = controller.mindMapScene
+        self.__nodemapCreator = self.__mindMapScene.nodeMapCreator
         self.__findText = None
         self.__caseSensitive = False
         self.__highlight = False
@@ -1370,11 +1440,12 @@ class NodeFinder(QObject):
         '''
         text = QTextDocument()
         text.setHtml(node.text)
+
         if self.__caseSensitive:
-            if self.__findText in text.toPlainText():
+            if unicode(self.__findText) in unicode(text.toPlainText()):
                 return True
         else:
-            if self.__findText.toLower() in text.toPlainText().toLower():
+            if unicode(self.__findText.toLower()) in unicode(text.toPlainText().toLower()):
                 return True
         return False
 
@@ -1432,8 +1503,9 @@ class NodeFinder(QObject):
         self.__findText = findText
         self.__caseSensitive = caseSensitive
         if findText is None or not len(findText): return
-        self.__finder.findNodes(self.__mindMapScene.rootDelegate.node, self._condition)
-        self.__foundNodes = self.__finder.createSortedList()
+        nodeMap = self.__nodemapCreator.createNodeMap(self.__mindMapScene.rootDelegate.node)
+        filteredNodeMap = self.__nodemapCreator.createFilteredNodeMap(nodeMap, self._condition)
+        self.__foundNodes = self.__nodemapCreator.createSortedNodeList(filteredNodeMap)
         if highlight: self.highlightNodes(highlight)
         self._status()
 
