@@ -93,6 +93,7 @@ class MainWindowController(QObject):
         self.__mindMapController = MindMapController(self.__mindMapScene, self.__actionBag)
         self.__networkController = hivemind.network.core.NetworkController(
                 self.__mindMapController, self.__actionBag)
+        self.__networkController.initialize()
         self.__smartConferenceRobot = SmartConferenceRobot(self.__mindMapScene,
                 self.__networkController, self.__actionBag)
         self.__actionBag.startSmartConferenceRobot.triggered.connect(
@@ -107,6 +108,8 @@ class MainWindowController(QObject):
         _configureApplication(self.__mindMapController)
         self.__findWidget = gui_factory.createFindToolBar(self)
         self.__filterWidget = gui_factory.createFilterToolBar(self)
+        self.__findWidget.setObjectName('findToolbar')
+        self.__filterWidget.setObjectName('filterToolbar')
         self._addToolbar(self.__filterWidget)
         self._addToolbar(self.__findWidget)
 
@@ -299,8 +302,8 @@ class MainWindowController(QObject):
     def _recordCommand(self, command):
         '''
         Process created command
-        
-        @param command: undo stack command to 
+
+        @param command: undo stack command to
         '''
         self.__networkController.publishCommand(command)
 
@@ -312,7 +315,7 @@ class MainWindowController(QObject):
         '''
         Handler of item click action
         @param rightButton: whether user used right button or left
-        @type rightButton: bool 
+        @type rightButton: bool
         @type item: NodeDelegate
         @type parent: NodeDelegate
         @param childLocation: location of child nodes
@@ -443,15 +446,16 @@ class MainWindowController(QObject):
 
     def onFind(self):
         '''Process find action'''
+        self.__filterWidget.enableFiltration(False)
         self.__filterWidget.setVisible(False)
-        self.__findWidget.setVisible(not self.__findWidget.isVisible())
         self.__findWidget.setDefaultFocus()
+        self.__findWidget.setVisible(not self.__findWidget.isVisible())
 
     def onFilter(self):
         '''Process filter action'''
         self.__findWidget.setVisible(False)
         self.__filterWidget.setVisible(not self.__filterWidget.isVisible())
-        self.__filterWidget.setDefaultFocus()
+        self.__filterWidget.enableFiltration(self.__filterWidget.isVisible())
 
     def editEdge(self):
         '''Edit node's edge decorations and style'''
@@ -925,10 +929,10 @@ class MindMapController(QObject):
         '''
         Show dialog to the user asking him/her whether there is a need to recover
         autosaved data copy or not
-        
+
         @param filename: name of the mind map file
         @type filename: str
-        
+
         @return: True if user want to recover data, False otherwise
         @rtype: bool
         '''
@@ -958,7 +962,7 @@ Options:
 def _configureApplication(controller):
     '''
     Parse command line arguments and configure application
-    
+
     @param controller: the mind map controller controller
     @type controller: MindMapController
     '''
