@@ -23,9 +23,11 @@
 '''
 Helper widgets
 '''
+from hivemind.core import Node
 import hivemind.settings as settings
 import hivemind.resource as resource
-import math
+import hivemind
+import math, sip
 from PyQt4.QtGui import QGraphicsView, QTextEdit, QGraphicsWidget, \
 QListWidgetItem, QListWidget, QGraphicsScene, QAbstractButton, QWidget, \
 QAction, QUndoCommand, QLineEdit, QSpinBox, QCheckBox, QComboBox, QLabel, \
@@ -421,7 +423,7 @@ class IconList(QGraphicsScene):
 class ActionImageButton(QAbstractButton):
     '''
     Command widget that only shows the icon of the action and tightly assosiated with it.
-    
+
     @author: Andrew Vasilev
     '''
 
@@ -459,7 +461,7 @@ class ActionImageButton(QAbstractButton):
 
     def _setupSizeHint(self, sizeHint):
         '''
-        Setup size hint attribute for the button. It cannot be greater than 
+        Setup size hint attribute for the button. It cannot be greater than
         ActionImageButton.__MAX_ICON_HEIGHT and the size of the icon.
         @param sizeHint: proposed height/width of the button
         @type sizeHint: int
@@ -482,7 +484,7 @@ class ActionImageButton(QAbstractButton):
         '''
         Paint action icon in the corresponding state
         @param paintEvent: paint event representation
-        @type paintEvent: QPaintEvent 
+        @type paintEvent: QPaintEvent
         '''
         painter = QPainter(self)
         if not self.__action.isEnabled(): mode = QIcon.Disabled
@@ -499,7 +501,8 @@ class StatusAction(QAction):
     '''
     readable('image')
 
-    def __init__(self, text, imageBaseName, status, parent = None):
+    #pylint: disable=R0913
+    def __init__(self, text, imageBaseName, status, parent = None, size = None):
         '''
         Constructs action
         @param text: Text of the action
@@ -515,6 +518,7 @@ class StatusAction(QAction):
         SIGNAL('imageChanged(QImage)')
         SIGNAL('statusChanged(str)')
         self.__imageBaseName = imageBaseName
+        self.__size = size
         self._setStatus(status)
 
     def _setStatus(self, status):
@@ -524,6 +528,8 @@ class StatusAction(QAction):
         #pylint: disable=W0201
         self.__status = status
         self.__image = resource.getImage('%s_%s' % (self.__imageBaseName, self.__status))
+        if self.__size:
+            self.__image = self.__image.scaled(self.__size, self.__size)
         icon = QIcon()
         icon.addPixmap(self.__image, QIcon.Disabled)
         self.setIcon(icon)
@@ -541,17 +547,17 @@ class StatusAction(QAction):
 class WidgetVisibilityAnimation(QAbstractAnimation):
     '''
     Widget visibility changer for the animation framework
-    
+
     @author: Andrew Vasilev
     '''
 
     def __init__(self, parent, visibility):
         '''
         Initialize animation
-        
+
         @param parent: target widget for visibility change
         @type parent: QWidget
-        
+
         @param visibility: the visibility value
         @param visibility: bool
         '''
@@ -1498,9 +1504,9 @@ class MaemoNodeMenu(QObject):
         '''
         @param angle: angle of the point in radians
         @type angle: float
-        
+
         @return: the point on ellipse, identified by angle
-        @rtype: float tuple 
+        @rtype: float tuple
         '''
         x = self.__basePoint.x() + self.__radiusOne * \
                 math.cos(angle + MaemoNodeMenu.__INITIAL_ANGLE)
@@ -1527,7 +1533,7 @@ class MaemoNodeMenu(QObject):
     def _isButtonInside(self, x, y):
         '''
         Check whether button fits the view if placed on (x, y) coordinates
-          
+
         @return: True if button fits on the view
         @rtype: bool
         '''
@@ -1541,12 +1547,12 @@ class MaemoNodeMenu(QObject):
         '''
         @param pointOne: first point coordinates
         @type pointOne: tuple
-        
+
         @param pointTwo: second point coordinates
         @type pointTwo: tuple
-        
+
         @return: distance between passed points
-        @rtype: float 
+        @rtype: float
         '''
         return math.sqrt((pointOne[0] - pointTwo[0]) ** 2
                          + (pointOne[1] - pointTwo[1]) ** 2)
@@ -1557,11 +1563,11 @@ class MaemoNodeMenu(QObject):
     def _positionMenuButtons(self, startAngle):
         '''
         Position menu buttons starting from the passed angle
-        
-        @param startAngle: base angle for positioning in radian measurement 
+
+        @param startAngle: base angle for positioning in radian measurement
         @type startAngle: float
-        
-        @return: 
+
+        @return:
         @rtype: float
         '''
         angle = startAngle
@@ -1599,13 +1605,13 @@ class MaemoNodeMenu(QObject):
         '''
         Calculate transitional position for animation between
         startPoint and endPoint.
-        
+
         @param startPoint: start point of animation
         @type startPoint: QRect
-        
+
         @param endPoint: end point of animation
         @type endPoint: QRect
-        
+
         @return: transition point to emulate twisting animation
         @rtype: QRect
         '''
@@ -1619,7 +1625,7 @@ class MaemoNodeMenu(QObject):
 
     def _generateAnimations(self):
         '''
-        Create and set up show and hide animations for menu buttons 
+        Create and set up show and hide animations for menu buttons
         '''
         self.__showAnimationTransformation = dict()
         self.__hideAnimationTransformation = dict()
@@ -1831,7 +1837,34 @@ class MaemoSidePanel(QObject):
         self.__hideAnimation.finished.connect(self._toggleSidebarStatus)
 
 
-class MaemoToolBar(QToolBar):
+class HivemindToolBar(QToolBar):
+    '''
+    Generic toolbar for handling events
+    @author: Oleg Kandaurov
+    '''
+
+    def __init__(self, parent = None):
+        '''
+        @type parent: QWidget
+        '''
+        QToolBar.__init__(self, parent)
+        parent.installEventFilter(self)
+
+    def eventFilter(self, obj, event):
+        '''
+        Catch events
+        @type obj: QEvent
+        @type event: QEvent
+        '''
+        if obj != self.parent():
+            return QToolBar.eventFilter(self, obj, event)
+        if event.type() == QEvent.KeyRelease:
+            if event.key() == Qt.Key_Escape:
+                self._close()
+        return QToolBar.eventFilter(self, obj, event)
+
+
+class MaemoToolBar(HivemindToolBar):
     '''
     Toolbar for Maemo platform
     @author: Eldar Mamedov, Oleg Kandaurov
@@ -1848,7 +1881,7 @@ class MaemoToolBar(QToolBar):
         '''
         @type parent: QWidget
         '''
-        QToolBar.__init__(self, parent)
+        HivemindToolBar.__init__(self, parent)
         self.resize(self.__WIDTH, self.__HEIGHT)
         self.setAllowedAreas(Qt.NoToolBarArea)
         self.setMovable(False)
@@ -1906,37 +1939,120 @@ class FilterToolBar(SearchingTrMixin, QObject):
         '''
         QObject.__init__(self)
         SearchingTrMixin.__init__(self)
-        self._finder = MindMapFilter(controller)
+        self._filter = MindMapFilter(controller)
         self._closeButton = QPushButton()
+        self.__node = Node()
+        self._iconDialog = hivemind.gui_factory.createIconDialog(self.__node)
+        self._iconDialog.accepted.connect(self._acceptIconDialog)
 
-    def setDefaultFocus(self):
-        '''Set default focus'''
-        self._findLineEdit.setFocus()
+    def _acceptIconDialog(self):
+        '''Handler of accepted signal of icon dialog'''
+        alteredAttributes = self._iconDialog.attributes()
+        self.__node.setAttributes(**alteredAttributes)
+        self._filter.icons = self.__node.icons
+        self._setIconMode()
+
+    def enableFiltration(self, isEnabled):
+        '''Enable filtration'''
+        self._filter.toggleFilter(isEnabled)
+        if isEnabled:
+            self._filterLineEdit.setFocus()
 
     def _createComponents(self):
         '''Create components'''
         #pylint: disable = W0201
-        self._findWidget = QWidget()
-        self._findLineEdit = QLineEdit()
-        self._findLineEdit.textEdited.connect(self._updateParams)
-        self._filterButton = QPushButton(self.tr('Filter'))
-        self._filterButton.setCheckable(True)
-        self._filterButton.toggled.connect(self._finder.toggleFilter)
+        self._filterWidget = QWidget()
+        self._iconSelectionButton = QPushButton(QIcon(resource.getImage('add')),
+                self.tr('Select icons'))
+        self._iconSelectionButton.clicked.connect(self._iconDialog.exec_)
+        self._modeButton = ActionImageButton(self._modeAction)
+        self._modeAction.triggered.connect(self._changeMode)
+        self._filterLineEdit = QLineEdit()
+        self._filterLineEdit.textEdited.connect(self._updateText)
         self._closeAction = QAction(self)
 
-    def _createLayout(self):
-        '''Create layout'''
-        mainLayout = QHBoxLayout()
-        mainLayout.addWidget(self._findLineEdit)
-        mainLayout.addWidget(self._filterButton)
-        mainLayout.addStretch()
-        mainLayout.addWidget(self._closeButton)
-        self._findWidget.setLayout(mainLayout)
+    def _setIconMode(self):
+        '''Set icon mode'''
+        self._modeAction.status = 'icon'
+        self._filter.mode = 'icon'
+        self.clearLayout(self._filterWidget)
+        self._filterWidget.setLayout(self._createIconModeLayout())
+        self._filter.refresh()
 
-    def _updateParams(self):
-        '''Update filtered nodes'''
-        self._finder.setTargetText(self._findLineEdit.text())
-        self._finder.refresh()
+    def _setTextMode(self):
+        '''Set text mode'''
+        self._modeAction.status = 'text'
+        self._filter.mode = 'text'
+        self.clearLayout(self._filterWidget)
+        self._filterWidget.setLayout(self._createTextModeLayout())
+        self._filter.refresh()
+
+    def _changeMode(self):
+        '''Process change of mode'''
+        if self._modeAction.status == 'text':
+            self._setIconMode()
+        else:
+            self._setTextMode()
+
+    @classmethod
+    def clearLayout(cls, widget):
+        '''Clear layout of the widget'''
+        if widget.layout() is not None:
+            oldLayout = widget.layout()
+            for i in reversed(range(oldLayout.count())):
+                widget = oldLayout.itemAt(i).widget()
+                if widget is not None:
+                    widget.setParent(None)
+            sip.delete(oldLayout)
+
+    def _createTextModeLayout(self):
+        '''
+        Creates layout for text filtration mode
+        @return: Layout
+        '''
+        layout = QHBoxLayout()
+        layout.addWidget(self._modeButton)
+        layout.addWidget(self._filterLineEdit)
+        layout.addStretch()
+        layout.addWidget(self._closeButton)
+        return layout
+
+    def _createIconModeLayout(self):
+        '''
+        Creates layout for text filtration mode
+        @return: Layout
+        '''
+        layout = QHBoxLayout()
+        labels = self._createIconLabels()
+        layout.addWidget(self._modeButton)
+        layout.addWidget(self._iconSelectionButton)
+        for label in labels:
+            layout.addWidget(label)
+        layout.addStretch()
+        layout.addWidget(self._closeButton)
+        return layout
+
+    def _createIconLabels(self):
+        '''
+        Create labels with icons on it
+        @return: list of QLabel
+        '''
+        labels = []
+        for icon in self.__node.icons:
+            label = QLabel()
+            label.setPixmap(QPixmap.fromImage(resource.getIconByName(icon)))
+            labels.append(label)
+        return labels
+
+    def _updateText(self):
+        '''Update text in filter'''
+        self._filter.text = self._filterLineEdit.text()
+        self._filter.refresh()
+
+    def _close(self):
+        '''Close toolbar'''
+        self.enableFiltration(False)
+        self.close()
 
 
 class FindToolBar(SearchingTrMixin, QObject):
@@ -1992,6 +2108,16 @@ class FindToolBar(SearchingTrMixin, QObject):
         @type status: str
         '''
         if status == 'none': self._statusLabel.setText('')
-        if status == 'not_found': self._statusLabel.setText(self.tr('No found nodes'))
+        if status == 'not_found':
+            self._statusLabel.setText(self.tr('No found nodes'))
+            palette = QPalette(self._findLineEdit.palette())
+            palette.setBrush(QPalette.Base, Qt.red)
+            self._findLineEdit.setPalette(palette)
+        else:
+            self._findLineEdit.setPalette(QPalette())
         if status == 'first': self._statusLabel.setText(self.tr('First found node'))
         if status == 'last': self._statusLabel.setText(self.tr('Last found node'))
+
+    def _close(self):
+        '''Close toolbar'''
+        self.close()
