@@ -33,7 +33,7 @@ from hivemind.dialogs import EditNodeDialog, EditLabelDialog, IconDialog, \
     AboutDialog, EditEdgeDialog, SettingsDialog, AffiliationDialog, NetworkSettingsDialog, \
     NetworkClientDialog, NetworkConnectionDialog
 from hivemind.gui_widgets import ScalingWidget, ActionImageButton, FindToolBar, \
-    FilterToolBar, MaemoToolBar
+    FilterToolBar, MaemoToolBar, StatusAction
 from hivemind.attribute import adaptCallable
 import hivemind.__version__ as __version__
 
@@ -567,19 +567,26 @@ class MaemoFilterToolBar(MaemoToolBar, FilterToolBar):
         '''
         FilterToolBar.__init__(self, controller)
         MaemoToolBar.__init__(self, parent)
-        FilterToolBar._createComponents(self)
+        self._createComponents()
         self._initComponents()
-        FilterToolBar._createLayout(self)
+
+    def _createComponents(self):
+        '''Create components'''
+        #pylint: disable=W0201
+        self._modeAction = StatusAction('Filtration mode', 'filtermode', 'text')
+        FilterToolBar._createComponents(self)
 
     def _initComponents(self):
         '''Init components'''
-        self._closeAction.triggered.connect(self.close)
+        self._closeAction.triggered.connect(self._close)
         self._closeAction.setIcon(QIcon(resource.getImage('find_close')))
         self._closeButton = ActionImageButton(self._closeAction)
-        self._findLineEdit.setStyleSheet("QLineEdit { border-width: 8px;"
+        self._iconSelectionButton.setText('')
+        self._filterLineEdit.setStyleSheet("QLineEdit { border-width: 8px;"
                  + " border-color: white; border-style: solid;"
                  + " border-radius: 12px; background: white; }")
-        self.addWidget(self._findWidget)
+        self._setTextMode()
+        self.addWidget(self._filterWidget)
 
 
 class MaemoFindToolBar(MaemoToolBar, FindToolBar):
