@@ -29,7 +29,7 @@ from twisted import internet
 from twisted.words.protocols.jabber import jid
 from wokkel.client import HybridAuthenticator
 from wokkel.ping import PingHandler
-from hivemind.attribute import readable
+from hivemind.attribute import readable, Observable
 from idavoll.backend import BackendService
 from PyQt4.QtCore import QObject
 from hivemind.commands import StateModificationCommand, CurrentNodeCommand, ShutdownCommand, \
@@ -70,54 +70,35 @@ class NetworkController(QObject):
         @type actionBag: ActionBag
         '''
         QObject.__init__(self)
-        self.__errorHandler = ErrorHandler(self)
+        self.__errorHandler = ErrorHandler()
         self.__transport = Transport(self.__errorHandler)
-        self.__mindMapController = mindMapController
-        self.__actionBag = actionBag
         self.__affiliationModel = AffiliationModel()
+        self.__clientDialog = gui_factory.createNetworkClientDialog(
+                self.__transport.rosterModel)
+        self.__protocol = Protocol(self.__transport, self.__errorHandler, mindMapController,
+                self.__affiliationModel)
+        self.__actionBag = actionBag
         self.__affiliationDialog = gui_factory.createAffiliationDialog(self.__affiliationModel,
                 self.__transport.rosterModel)
         self.__networkDialog = gui_factory.createNetworkConnectionDialog()
-        self.__clientDialog = gui_factory.createNetworkClientDialog(
-                self.__transport.rosterModel)
         self.__networkSettingsDialog = gui_factory.createNetworkSettingsDialog()
-
-        self.__isServiceRole = False
-        self.__protocol = None
-        self.__pingHandler = None
-        self.__storage = None
         self.__secondConnectionStep = None
-        self.__clientRole = None
-        self.__serverPublishModel = 'publishers'
         self._connectActions()
         self.updateActions(connectionState.disconnected)
-        settings.connect(self._updateConfiguration)
+        settings.connect(self.__protocol.updateConfiguration)
+
+    def initialize(self):
+        '''Initialize controller'''
+        self.__transport.addObserver(self)
+        self.__protocol.addObserver(self)
+        self.__errorHandler.addObserver(self)
 
     def publishCommand(self, command):
         '''
         Publish command in common space
         @param command: command to publish in shared space
         '''
-        def isClientPropagationAllowed():
-            '''Whether client can publish or not'''
-            if (self.__clientRole in ['outcast', 'member'] and
-                    self.__serverPublishModel == 'publishers'):
-                return False
-            return True
-
-        if self.__protocol:
-            if self.__isServiceRole == True:
-                if _isPropagationAllowed(command):
-                    self.__protocol.proposeChangeset(
-                        Changeset(parser.serializeCommand(command), 'command'))
-                else:
-                    self.__mindMapController.executeCommand(command, False)
-            else:
-                if isClientPropagationAllowed():
-                    self.__protocol.proposeChangeset(
-                        Changeset(parser.serializeCommand(command), 'command'))
-        else:
-            self.__mindMapController.executeCommand(command)
+        self.__protocol.publishCommand(command)
 
     def terminateNetworkConnection(self):
         '''Terminate network connection'''
@@ -125,7 +106,7 @@ class NetworkController(QObject):
             '''Stop reactor'''
             if reactor.running:
                 reactor.stop()
-        if not self.__mindMapController.prohibitChangeMapAction():
+        if not self.__protocol.mindMapController.prohibitChangeMapAction():
             return
         reactor.callLater(SHUTDOWN_TIMEOUT, stopReactor, self)
         defer = self.closeNetwork()
@@ -148,8 +129,14 @@ class NetworkController(QObject):
             else:
                 self.__networkDialog.exec_()
 
+        def startClient():
+            '''Configure and start client'''
+            self.__protocol.configureClient(
+                    self.__clientDialog.attributes['serviceJid'] + '/hivemind')
+            self.__protocol.startProtocol()
+
         self.__actionBag.disconnectXMPPAction.triggered.connect(self.closeNetwork)
-        self.__actionBag.stopProtocolAction.triggered.connect(self.closeProtocol)
+        self.__actionBag.stopProtocolAction.triggered.connect(self.__protocol.close)
         self.__actionBag.startServiceAction.triggered.connect(prepareService)
         self.__actionBag.startServiceAction.triggered.connect(showNetworkDialog)
         self.__actionBag.startClientAction.triggered.connect(prepareClient)
@@ -159,8 +146,8 @@ class NetworkController(QObject):
         self.__actionBag.networkSettingsAction.triggered.connect(
                 self.__networkSettingsDialog.exec_)
         self.__networkDialog.accepted.connect(self._startNetwork)
-        self.__clientDialog.accepted.connect(self._startClient)
-        self.__affiliationDialog.accepted.connect(self._updateAffiliationsFromModel)
+        self.__clientDialog.accepted.connect(startClient)
+        self.__affiliationDialog.accepted.connect(self.__protocol.updateAffiliationsFromModel)
 
     def updateActions(self, state):
         '''
@@ -175,207 +162,69 @@ class NetworkController(QObject):
         self.__actionBag.startServiceAction.setDisabled(state == connectionState.collaboration)
         self.__actionBag.startClientAction.setDisabled(state == connectionState.collaboration)
         self.__actionBag.editAffiliationAction.setEnabled(
-                state == connectionState.collaboration and self.__isServiceRole)
+                state == connectionState.collaboration and self.__protocol.isServiceRole)
         self.__actionBag.networkSettingsAction.setDisabled(
-                state == connectionState.collaboration and not self.__isServiceRole)
-
-    def _updateConfiguration(self):
-        '''Set node configuration according to settings'''
-        if self.__storage is None:
-            return
-        config = {'pubsub#access_model': settings.get('accessModel'),
-                   'pubsub#publish_model': settings.get('publishModel')}
-        defer = self.__storage.getNode(XMPP_NODE_NAME)
-        defer.addCallback(lambda node: node.setConfiguration(config))
-
-    def _updateAffiliationsFromModel(self):
-        '''Update affiliations in node'''
-        affiliations = self.__affiliationModel.affiliations
-        for entity, affiliation in affiliations.iteritems():
-            defer = self.__storage.getNode(XMPP_NODE_NAME)
-            defer.addCallback(lambda node:
-                    node.setAffiliation(jid.internJID(entity), affiliation))
-            command = AffiliationCommand(entity, affiliation)
-            self.__protocol.proposeChangeset(
-                    Changeset(parser.serializeCommand(command), 'command'))
+                state == connectionState.collaboration and not self.__protocol.isServiceRole)
 
     def _startNetwork(self):
         '''Start network connection'''
-        self.__transport.setController(self)
         self.__transport.start(self.__networkDialog.attributes['userJid'] + '/hivemind',
                 self.__networkDialog.attributes['password'],
                 self.__networkDialog.attributes['host'],
                 self.__networkDialog.attributes['port'])
         reactor.callLater(2, self._performSecondConnectionStep)
 
-    def _startClient(self):
-        '''Start client'''
-        self._configureClient(self.__clientDialog.attributes['serviceJid'] + '/hivemind')
-        self._startProtocol()
-
-    def _startService(self):
-        '''Start service'''
-        self._configureService()
-        self._startProtocol()
-
     def _performSecondConnectionStep(self):
         ''' Perform second connection step'''
         if self.__secondConnectionStep and self.__transport.xmppClient is not None:
             {'client': self.__clientDialog.exec_,
-                        'service': self._startService}[self.__secondConnectionStep]()
+                        'service': self.__protocol.startService}[self.__secondConnectionStep]()
         self.__secondConnectionStep = None
 
-    def _configureService(self):
-        '''Configure HiveMind service'''
-        self.__isServiceRole = True
-        self.__storage = HivemindStorage()
-        backend = BackendService(self.__storage)
-        resource = HivemindResourceFromBackend(backend)
-        self.__protocol = HivemindPubSubService(resource)
-        resource.pubsubService = self.__protocol
-        resource.serviceJID = self.__transport.xmppClient.jid
-        mindMap = parser.mindMapToString(self.__mindMapController.initialMindMap, False)
-        undoStack = parser.serializeCommandStack(self.__mindMapController.undoStack)
-        backend.createNode(XMPP_NODE_NAME, resource.serviceJID)
-        self._updateConfiguration()
-        defer = self.__storage.getNode(XMPP_NODE_NAME)
-        defer.addCallback(lambda node: (node.addObserver(self),
-                node.storeChangeset(Changeset(mindMap, 'mindmap'), resource.serviceJID),
-                node.storeChangeset(Changeset(undoStack, 'undostack'), resource.serviceJID)))
-
-    def _configureClient(self, service):
-        '''
-        Configure HiveMind client
-        @param service: Service to connect to
-        @type service: str
-        '''
-        self.__isServiceRole = False
-        self.__protocol = HivemindClient(jid.internJID(service))
-
-    def _startProtocol(self):
-        '''Start Hivemind protocol'''
-        self.__protocol.addObserver(self)
-        self.__protocol.setHandlerParent(self.__transport.xmppClient)
-
-    def _startProtocolHelpers(self):
-        '''Start protocol helpers'''
-        self.__pingHandler = PingHandler()
-        self.__pingHandler.setHandlerParent(self.__transport.xmppClient)
-
     def closeNetwork(self):
         '''Closes network connection'''
-        defer = self.closeProtocol()
+        defer = self.__protocol.close()
         defer.addCallback(lambda success: self.__transport.close())
         return defer
 
-    def closeProtocol(self):
-        '''Close protocol level'''
-        def destroyProtocol(result):
-            '''Destroy protocol'''
-            if self.__transport.xmppClient:
-                self.__protocol.disownHandlerParent(self.__transport.xmppClient)
-            self.__protocol.deleteObserver(self)
-            self.__protocol = None
-            return result
-        if self.__transport is None: return
-        if self.__transport.xmppClient:
-            if self.__pingHandler:
-                self.__pingHandler.disownHandlerParent(self.__transport.xmppClient)
-                self.__pingHandler = None
-            if self.__transport.pingManager:
-                self.__transport.pingManager.removeAll([self.__transport.xmppClient.server])
-        defer = internet.defer.succeed(None)
-        if self.__protocol is None or self.__transport.xmppClient is None:
-            return defer
-        if self.__isServiceRole:
-            defer = self.__protocol.proposeChangeset(
-                    Changeset(parser.serializeCommand(ShutdownCommand()), 'command'))
-        defer.addBoth(self.__protocol.close)
-        defer.addBoth(destroyProtocol)
-        return defer
-
     def processNotification(self, *values, **kvargs):
         '''Process notification'''
         for event in kvargs.keys():
             getattr(self, '_process%s%sEvent' % (event[0].upper(), event[1:]))(kvargs[event])
 
-    def _processErrorEvent(self, failure):
-        '''Process error event'''
-        self.__errorHandler.mapErrorEvent(failure)
+    def _processActionEvent(self, action):
+        '''Process action event'''
+        mapAction = {
+                'finishTeamwork': self.__protocol.close,
+                'finishAll': self.closeNetwork}
+        performAction = mapAction[action]
+        performAction()
 
-    def _processAffiliationEvent(self, event):
-        '''Process affiliation event from node'''
-        entity, affiliation = event
-        self.__affiliationModel.updateAffiliation(entity, affiliation)
-
-    def _processProtocolPingEvent(self, event):
-        '''Process ping event from protocol'''
-        entity = event['entity']
-        state = event['state']
-        if state == Pinger.entityState.timeout:
-            if self.__protocol and self.__transport.xmppClient:
-                # seems broken, need inspection
-                self.__protocol.unsubscribe(entity, self.__transport.xmppClient.jid,
-                            XMPP_NODE_NAME, entity)
-            if self.__transport.pingManager:
-                self.__transport.pingManager.stopPing(entity)
-
-    def _processSubscriptionEvent(self, event):
-        '''Process subscription event'''
-        entity = event['entity']
-        state = event['state']
-        self.__affiliationModel.updateSubscriptionState(entity, state)
-        if self.__transport.pingManager is None: return
-        if state == subscriptionState.subscribed:
-            self.__transport.pingManager.addPinger(entity,
-                    settings.get('protocolPingInterval'),
-                    settings.get('protocolPingMaxCount'))
-            self.__transport.pingManager.startPing(entity)
-        else:
-            self.__transport.pingManager.removePinger(entity)
-
-    def _processConnectionEvent(self, state):
-        '''Process connection event'''
-        mapConnectionState = {connectionState.initialized: self.__transport.startHelpers,
-                connectionState.collaboration:  self._startProtocolHelpers}
+    def _processStateEvent(self, stateInfo):
+        '''Process connection state'''
+        mapConnectionState = {
+                connectionState.initialized: self.__transport.startHelpers,
+                connectionState.disconnected: self.closeNetwork,
+                connectionState.connecting: None,
+                connectionState.connected: None,
+                connectionState.collaboration:  self.__protocol.startProtocolHelpers}
+        state, perform = stateInfo
         processEvent = mapConnectionState[state]
-        if processEvent is not None:
+        if processEvent is not None and perform:
             processEvent()
         self.updateActions(state)
 
-    def _processChangesetEvent(self, changeset):
-        '''Process changeset event'''
-        if (self.__isServiceRole and changeset.type != 'command'):
-            return
-        data = self.__mindMapController.deserializeData(changeset.data, changeset.type)
-        if not self.__isServiceRole:
-            if isinstance(data, ShutdownCommand):
-                self.closeProtocol()
-                return
-            elif isinstance(data, AffiliationCommand):
-                if data.entity == self.__transport.xmppClient.jid.userhost():
-                    self.__clientRole = data.affiliation
-                return
-        mapChangesetType = {'mindmap': 'loadMindMap',
-                'command': 'executeCommand',
-                'undostack': 'loadUndoStack'}
-        processChangeset = mapChangesetType[changeset.type]
-        getattr(self.__mindMapController, processChangeset)(data)
 
-
-class ErrorHandler(QObject):
+class ErrorHandler(QObject, Observable):
     '''
     Handles network errors
     @author: Oleg Kandaurov
     '''
 
-    def __init__(self, networkCtrl):
-        """
-        Constructor
-        @type networkCtrl: NetworkController
-        """
+    def __init__(self):
+        '''Constructor'''
         QObject.__init__(self)
-        self.__networkCtrl = networkCtrl
+        Observable.__init__(self)
         self.__streamMessages = messages.StreamErrorMessages()
         self.__saslMessages = messages.SASLErrorMessages()
         self.__stanzaMessages = messages.StanzaErrorMessages()
@@ -386,7 +235,8 @@ class ErrorHandler(QObject):
         Handle sasl error
         @type failure: Failure
         '''
-        errorDialog = gui_factory.createNetworkErrorDialog(self.__networkCtrl.closeNetwork)
+        errorDialog = gui_factory.createNetworkErrorDialog('finishAll')
+        errorDialog.performAction.connect(self._notify)
         errorDialog.showError(self.tr('Authentication fails.'),
                 self.tr('Seems that you provide invalid username or password. ' +
                 'See details for more info.'),
@@ -397,7 +247,8 @@ class ErrorHandler(QObject):
         Handle stream error
         @type failure: Failure
         '''
-        errorDialog = gui_factory.createNetworkErrorDialog(self.__networkCtrl.closeNetwork)
+        errorDialog = gui_factory.createNetworkErrorDialog('finishAll')
+        errorDialog.performAction.connect(self._notify)
         errorDialog.showError(self.tr('XMPP connection fails.'),
                 self.tr('The error is nonrecoverable. ' +
                 'You are automatically disconnected. ' +
@@ -409,7 +260,8 @@ class ErrorHandler(QObject):
         Handle stanza error
         @type failure: Failure
         '''
-        errorDialog = gui_factory.createNetworkErrorDialog(self.__networkCtrl.closeProtocol)
+        errorDialog = gui_factory.createNetworkErrorDialog('finishTeamwork')
+        errorDialog.performAction.connect(self._notify)
         errorDialog.showError(self.tr('Collaboration error has been occured.'),
                 self.tr('Fortunately, the error is not fatal. ' +
                 'Often this means that you are not allowed to do some action. ' +
@@ -421,7 +273,8 @@ class ErrorHandler(QObject):
         Handle pubsub error
         @type failure: Failure
         '''
-        errorDialog = gui_factory.createNetworkErrorDialog(self.__networkCtrl.closeProtocol)
+        errorDialog = gui_factory.createNetworkErrorDialog('finishTeamwork')
+        errorDialog.performAction.connect(self._notify)
         condition, feature = protocol_layer.pubsubConditionFromStanzaError(failure.value)
         message = self.__stanzaMessages.condition[failure.value.condition] + \
                 self.tr('\nCondition: %s') % (
@@ -435,6 +288,10 @@ class ErrorHandler(QObject):
                 'See details for more info.'),
                 message, False)
 
+    def _notify(self, action):
+        '''Notify observers'''
+        self.notifyObservers(action = str(action))
+
     def mapErrorEvent(self, failure):
         ''''
         Map different network errors
@@ -453,28 +310,227 @@ class ErrorHandler(QObject):
                 self._handleStanzaError(failure)
 
 
-class Transport(object):
+class Protocol(Observable):
+    '''
+    Manages protocol handlers
+
+    @author: Oleg Kandaurov
+    '''
+
+    readable('mindMapController', 'isServiceRole')
+
+    def __init__(self, transport, errorHandler, mindMapController, affiliationModel):
+        '''Constructor'''
+        Observable.__init__(self)
+        self.__transport = transport
+        self.__errorHandler = errorHandler
+        self.__mindMapController = mindMapController
+        self.__affiliationModel = affiliationModel
+        self.__isServiceRole = False
+        self.__oldProtocol = None
+        self.__pingHandler = None
+        self.__pingManager = None
+        self.__storage = None
+        self.__clientRole = None
+
+    def publishCommand(self, command):
+        '''
+        Publish command in common space
+        @param command: command to publish in shared space
+        '''
+        def isClientPropagationAllowed():
+            '''Whether client can publish or not'''
+            if (self.__clientRole in ['outcast', 'member'] and
+                    self.__serverPublishModel == 'publishers'):
+                return False
+            return True
+
+        if self.__oldProtocol:
+            if self.__isServiceRole == True or isClientPropagationAllowed():
+                if _isPropagationAllowed(command):
+                    self.__oldProtocol.proposeChangeset(
+                        Changeset(parser.serializeCommand(command), 'command'))
+                else:
+                    self.__mindMapController.executeCommand(command, False)
+        else:
+            self.__mindMapController.executeCommand(command)
+
+    def updateConfiguration(self):
+        '''Set node configuration according to settings'''
+        if self.__storage is None:
+            return
+        config = {'pubsub#access_model': settings.get('accessModel'),
+                   'pubsub#publish_model': settings.get('publishModel')}
+        defer = self.__storage.getNode(XMPP_NODE_NAME)
+        defer.addCallback(lambda node: node.setConfiguration(config))
+
+    def configureService(self):
+        '''Configure HiveMind service'''
+        self.__isServiceRole = True
+        self.__storage = HivemindStorage()
+        backend = BackendService(self.__storage)
+        resource = HivemindResourceFromBackend(backend)
+        self.__oldProtocol = HivemindPubSubService(resource)
+        resource.pubsubService = self.__oldProtocol
+        resource.serviceJID = self.__transport.xmppClient.jid
+        mindMap = parser.mindMapToString(self.__mindMapController.initialMindMap, False)
+        undoStack = parser.serializeCommandStack(self.__mindMapController.undoStack)
+        backend.createNode(XMPP_NODE_NAME, resource.serviceJID)
+        self.updateConfiguration()
+        defer = self.__storage.getNode(XMPP_NODE_NAME)
+        defer.addCallback(lambda node: (node.addObserver(self),
+                node.storeChangeset(Changeset(mindMap, 'mindmap'), resource.serviceJID),
+                node.storeChangeset(Changeset(undoStack, 'undostack'), resource.serviceJID)))
+
+    def configureClient(self, service):
+        '''
+        Configure HiveMind client
+        @param service: Service to connect to
+        @type service: str
+        '''
+        self.__isServiceRole = False
+        self.__oldProtocol = HivemindClient(jid.internJID(service))
+
+    def startProtocol(self):
+        '''Start Hivemind protocol'''
+        self.__oldProtocol.addObserver(self)
+        self.__oldProtocol.setHandlerParent(self.__transport.xmppClient)
+
+    def close(self):
+        '''Close protocol level'''
+        def destroyProtocol(result):
+            '''Destroy protocol'''
+            if self.__transport.xmppClient:
+                self.__oldProtocol.disownHandlerParent(self.__transport.xmppClient)
+            self.__oldProtocol.deleteObserver(self)
+            self.__oldProtocol = None
+            return result
+        if self.__transport is None: return
+        if self.__transport.xmppClient:
+            if self.__pingHandler:
+                self.__pingHandler.disownHandlerParent(self.__transport.xmppClient)
+                self.__pingHandler = None
+        self._destroyPingManager()
+        defer = internet.defer.succeed(None)
+        if self.__oldProtocol is None or self.__transport.xmppClient is None:
+            return defer
+        if self.__isServiceRole:
+            defer = self.__oldProtocol.proposeChangeset(
+                    Changeset(parser.serializeCommand(ShutdownCommand()), 'command'))
+        defer.addBoth(self.__oldProtocol.close)
+        defer.addBoth(destroyProtocol)
+        return defer
+
+    def _destroyPingManager(self):
+        '''Destroy ping manager'''
+        if self.__pingManager:
+            if self.__transport.xmppClient:
+                self.__pingManager.removeAll()
+            self.__pingManager = None
+
+    def startService(self):
+        '''Start service'''
+        self.configureService()
+        self.startProtocol()
+
+    def startProtocolHelpers(self):
+        '''Start protocol helpers'''
+        self.__pingManager = PingManager(self, self.__transport.xmppClient)
+        self.__pingHandler = PingHandler()
+        self.__pingHandler.setHandlerParent(self.__transport.xmppClient)
+
+    def updateAffiliationsFromModel(self):
+        '''Update affiliations in node'''
+        affiliations = self.__affiliationModel.affiliations
+        for entity, affiliation in affiliations.iteritems():
+            defer = self.__storage.getNode(XMPP_NODE_NAME)
+            defer.addCallback(lambda node:
+                    node.setAffiliation(jid.internJID(entity), affiliation))
+            command = AffiliationCommand(entity, affiliation)
+            self.__oldProtocol.proposeChangeset(
+                    Changeset(parser.serializeCommand(command), 'command'))
+
+    def processNotification(self, *values, **kvargs):
+        '''Process notification'''
+        for event in kvargs.keys():
+            getattr(self, '_process%s%sEvent' % (event[0].upper(), event[1:]))(kvargs[event])
+
+    def _processAffiliationEvent(self, event):
+        '''Process affiliation event from node'''
+        entity, affiliation = event
+        self.__affiliationModel.updateAffiliation(entity, affiliation)
+
+    def _processErrorEvent(self, failure):
+        '''Process error event'''
+        self.__errorHandler.mapErrorEvent(failure)
+
+    def _processProtocolPingEvent(self, event):
+        '''Process ping event from protocol'''
+        entity = event['entity']
+        state = event['state']
+        if state == Pinger.entityState.timeout:
+            if self.__oldProtocol and self.__transport.xmppClient:
+                # seems broken, need inspection
+                self.__oldProtocol.unsubscribe(entity, self.__transport.xmppClient.jid,
+                            XMPP_NODE_NAME, entity)
+            if self.__pingManager:
+                self.__pingManager.stopPing(entity)
+
+    def _processSubscriptionEvent(self, event):
+        '''Process subscription event'''
+        entity = event['entity']
+        state = event['state']
+        self.__affiliationModel.updateSubscriptionState(entity, state)
+        if self.__pingManager is None: return
+        if state == subscriptionState.subscribed:
+            self.__pingManager.addPinger(entity,
+                    settings.get('protocolPingInterval'),
+                    settings.get('protocolPingMaxCount'))
+            self.__pingManager.startPing(entity)
+        else:
+            self.__pingManager.removePinger(entity)
+
+    def _processConnectionEvent(self, state):
+        '''Process connection event'''
+        self.notifyObservers(state = (state, True))
+
+    def _processChangesetEvent(self, changeset):
+        '''Process changeset event'''
+        if self.__isServiceRole and changeset.type != 'command':
+            return
+        data = self.__mindMapController.deserializeData(changeset.data, changeset.type)
+        if not self.__isServiceRole:
+            if isinstance(data, ShutdownCommand):
+                self.close()
+                return
+            elif isinstance(data, AffiliationCommand):
+                if data.entity == self.__transport.xmppClient.jid.userhost():
+                    self.__clientRole = data.affiliation
+                return
+        mapChangesetType = {'mindmap': 'loadMindMap',
+                'command': 'executeCommand',
+                'undostack': 'loadUndoStack'}
+        processChangeset = mapChangesetType[changeset.type]
+        getattr(self.__mindMapController, processChangeset)(data)
+
+
+class Transport(Observable):
     '''
     Manages XMPP connection
 
     @author: Oleg Kandaurov
     '''
 
-    readable('xmppClient', 'pingManager', 'rosterModel')
+    readable('xmppClient', 'rosterModel')
     def __init__(self, errorHandler):
         '''Constructor'''
+        Observable.__init__(self)
         self.__errorHandler = errorHandler
         self.__rosterModel = RosterModel()
-        self.__controller = None
         self.__rosterProtocol = None
         self.__pinger = None
-        self.__pingManager = None
         self.__xmppClient = None
 
-    def setController(self, controller):
-        '''Setter'''
-        self.__controller = controller
-
     def start(self, userJid, password, host, port):
         '''
         Start XMPP connection
@@ -492,12 +548,14 @@ class Transport(object):
         self.__xmppClient.addObserver(self)
         self.__xmppClient.startService()
 
-    def startHelpers(self):
-        '''Start transport helpers'''
+    def _createRoster(self):
+        '''Create roster'''
         self.__rosterProtocol = HivemindRosterClientProtocol()
         self.__rosterProtocol.addObserver(self)
         self.__rosterProtocol.setHandlerParent(self.__xmppClient)
-        self.__pingManager = PingManager(self, self.__xmppClient)
+
+    def _createPinger(self):
+        '''Create pinger'''
         self.__pinger = Pinger(self.__xmppClient.server,
                 settings.get('transportPingInterval'),
                 settings.get('transportPingMaxCount'), isServer = True)
@@ -505,26 +563,36 @@ class Transport(object):
         self.__pinger.addObserver(self)
         self.__pinger.startPing()
 
+    def startHelpers(self):
+        '''Start transport helpers'''
+        self._createRoster()
+        self._createPinger()
+
     def close(self):
         ''' Close transport level'''
+        self._destroyRoster()
+        self._destroyPinger()
+        if self.__xmppClient:
+            self.__xmppClient.stopService()
+            self.__xmppClient = None
+        self.notifyObservers(state = (connectionState.disconnected, False) )
+
+    def _destroyRoster(self):
+        '''Destroy roster protocol'''
         if self.__rosterProtocol:
             if self.__xmppClient:
                 self.__rosterProtocol.disownHandlerParent(self.__xmppClient)
             self.__rosterProtocol.deleteObserver(self)
             self.__rosterProtocol = None
-        if self.__pingManager:
-            if self.__xmppClient:
-                self.__pingManager.removeAll()
-                self.__pingManager = None
+
+    def _destroyPinger(self):
+        '''Destroy pinger'''
         if self.__pinger:
             self.__pinger.stopPing()
-            self.__pinger.disownHandlerParent(self.__xmppClient)
+            if self.__xmppClient:
+                self.__pinger.disownHandlerParent(self.__xmppClient)
+            self.__pinger.deleteObserver(self)
             self.__pinger = None
-        if self.__xmppClient:
-            self.__xmppClient.stopService()
-            self.__xmppClient = None
-        if self.__controller is not None:
-            self.__controller.updateActions(connectionState.disconnected)
 
     def processNotification(self, *values, **kvargs):
         '''Process notification'''
@@ -555,14 +623,7 @@ class Transport(object):
 
     def _processConnectionEvent(self, state):
         '''Process connection event'''
-        mapConnectionState = {connectionState.initialized: self.startHelpers,
-                              connectionState.disconnected: self.__controller.closeNetwork,
-                              connectionState.connecting: None,
-                              connectionState.connected: None}
-        processEvent = mapConnectionState[state]
-        if processEvent is not None:
-            processEvent()
-        self.__controller.updateActions(state)
+        self.notifyObservers(state = (state, True))
 
     def _reconnect(self):
         '''
