@@ -30,9 +30,12 @@ configuration.
 @author: Ilya Paramonov
 '''
 
-import os
+import os, sys
 
-BASEDIR = os.path.dirname(os.path.dirname(__file__))
+if os.name == 'nt':
+    BASEDIR = os.path.dirname(sys.executable)
+else:
+    BASEDIR = os.path.dirname(os.path.dirname(__file__))
 '''
 Base directory of application.
 
